#
# (c) Jan Gehring <jan.gehring@gmail.com>
#
# vim: set ts=2 sw=2 tw=0:
# vim: set expandtab:

package Rex::Args::Single;

use 5.010001;
use strict;
use warnings;

our $VERSION = '1.13.4'; # VERSION

sub get { return 1; }

1;
